@@ -957,14 +957,31 @@ spGP.Gibbs<-function(formula, data=parent.frame(), time.data, coords,
           cat("##", "\n")
           cat("# Predicted samples and summary statistics are given.\n# nBurn = ",nBurn+posteriors$nBurn,". Iterations = ",posteriors$iterations,".", "\n")
           cat("##", "\n")
-          #
+      	  if(posteriors$model == "truncated"){
+             #output$pred.samples <- reverse.truncated.fnc(output$pred.samples,at=posteriors$truncation.para$at,lambda=posteriors$truncation.para$lambda)
+	         output$prob.below.threshold <- prob.below.threshold(output$pred.samples, at=posteriors$truncation.para$at)
+             szp<-spT.Summary.Stat(output$pred.samples[,])
+             output$Mean <- matrix(szp$Mean,rT, nsite)
+			 output$Mean <- reverse.truncated.fnc(output$Mean,at=posteriors$truncation.para$at,lambda=posteriors$truncation.para$lambda)
+             output$Median <- matrix(szp$Median,rT, nsite)
+			 output$Median <- reverse.truncated.fnc(output$Median,at=posteriors$truncation.para$at,lambda=posteriors$truncation.para$lambda)
+             output$SD <- matrix(szp$SD,rT, nsite)
+             output$Low <- matrix(szp[,4],rT, nsite)
+             output$Up <- matrix(szp[,5],rT, nsite)
+             szp <- NULL
+			 output$Low <- reverse.truncated.fnc(output$Low,at=posteriors$truncation.para$at,lambda=posteriors$truncation.para$lambda)
+ 			 output$Up <- reverse.truncated.fnc(output$Up,at=posteriors$truncation.para$at,lambda=posteriors$truncation.para$lambda)
+    	     output$truncation.para <- posteriors$truncation.para
+	      }
+		  else{
           szp<-spT.Summary.Stat(output$pred.samples[,])
-          # 
           output$Mean <- matrix(szp$Mean,rT, nsite)
           output$Median <- matrix(szp$Median,rT, nsite)
           output$SD <- matrix(szp$SD,rT, nsite)
           output$Low <- matrix(szp[,4],rT, nsite)
           output$Up <- matrix(szp[,5],rT, nsite)
+          szp <- NULL
+		  }
     #
    end.time <- proc.time()[3]
    comp.time<-end.time-start.time
@@ -1168,16 +1185,6 @@ spGP.Gibbs<-function(formula, data=parent.frame(), time.data, coords,
       else{
            rhotp<-posteriors$rhotp[,(nBurn+1):nItr]
       }
-      #out<-matrix(.C('zlt_fore_gp_tp_its',as.integer(cov),
-      #     as.integer(itt),as.integer(K),as.integer(nsite),as.integer(n),
-      #     as.integer(newr),as.integer(p),as.integer(u),as.integer(newr*T),as.integer(T),
-      #     as.integer(newr*K),as.integer(nsite*newr*K), 
-      #     as.double(coords.D),as.double(t(coords.f.D)),as.double(phip),as.double(nup),
-      #     as.double(sig_ep),as.double(sig_etap),as.double(sig_deltap),as.double(sig_op),
-      #     as.double(fore.x),as.double(fore.xtp),
-      #     as.double(betap),as.double(rhotp),as.double(betat0p),as.double(betatp),
-      #     as.double(w),as.integer(1),
-      #     foreZ=double(nsite*newr*K*itt))$foreZ,nsite*newr*K,itt)
       out<-.C('zlt_fore_gp_tp_its',as.integer(cov),
            as.integer(itt),as.integer(K),as.integer(nsite),as.integer(n),
            as.integer(newr),as.integer(p),as.integer(u),as.integer(newr*T),as.integer(T),
@@ -1272,7 +1279,24 @@ spGP.Gibbs<-function(formula, data=parent.frame(), time.data, coords,
           cat("##", "\n")
           cat("# Forecast samples and summary statistics are given.\n# nBurn = ",nBurn,". Iterations = ",nItr,".", "\n")
           cat("##", "\n")
-          #
+          # 
+      	  if(posteriors$model == "truncated"){
+		     #output$fore.samples <- reverse.truncated.fnc(output$fore.samples,at=posteriors$truncation.para$at,lambda=posteriors$truncation.para$lambda)
+	         output$prob.below.threshold <- prob.below.threshold(output$fore.samples[,], at=posteriors$truncation.para$at)
+             szp<-spT.Summary.Stat(output$fore.samples[,])
+             output$Mean <- matrix(szp$Mean,newr*K, nsite)
+             output$Median <- matrix(szp$Median,newr*K, nsite)
+             output$SD <- matrix(szp$SD,newr*K, nsite)
+             output$Low <- matrix(szp[,4],newr*K, nsite)
+             output$Up <- matrix(szp[,5],newr*K, nsite)
+			 output$Mean <- reverse.truncated.fnc(output$Mean,at=posteriors$truncation.para$at,lambda=posteriors$truncation.para$lambda)
+ 			 output$Median <- reverse.truncated.fnc(output$Median,at=posteriors$truncation.para$at,lambda=posteriors$truncation.para$lambda)
+			 output$Low <- reverse.truncated.fnc(output$Low,at=posteriors$truncation.para$at,lambda=posteriors$truncation.para$lambda)
+ 			 output$Up <- reverse.truncated.fnc(output$Up,at=posteriors$truncation.para$at,lambda=posteriors$truncation.para$lambda)
+             output$fore.samples <- reverse.truncated.fnc(output$fore.samples,at=posteriors$truncation.para$at,lambda=posteriors$truncation.para$lambda)
+    	     output$truncation.para <- posteriors$truncation.para
+	      }
+		  else{
           szp<-spT.Summary.Stat(output$fore.samples[,])
           # 
           output$Mean <- matrix(szp$Mean,newr*K, nsite)
@@ -1280,6 +1304,8 @@ spGP.Gibbs<-function(formula, data=parent.frame(), time.data, coords,
           output$SD <- matrix(szp$SD,newr*K, nsite)
           output$Low <- matrix(szp[,4],newr*K, nsite)
           output$Up <- matrix(szp[,5],newr*K, nsite)
+		  }
+	   ##
     #
    end.time <- proc.time()[3]
    comp.time<-end.time-start.time
@@ -1292,18 +1318,22 @@ spGP.Gibbs<-function(formula, data=parent.frame(), time.data, coords,
       }
 }
 ##
-## MCMC fit and predictions for the GP models
+## MCMC sampling for the truncated GP models
+## time.data format: col-1: year, col-2: month, col-3: day
 ##
-spGP.MCMC.Pred<-function(formula, data=parent.frame(), time.data, 
-            coords, pred.coords, priors, initials, pred.data, nItr, nBurn, 
-			report=1, tol.dist=2, distance.method="geodetic:km", cov.fnc="exponential",
-            scale.transform="NONE", spatial.decay, annual.aggregation="NONE")
+sptruncated.Gibbs<-function(formula, data=parent.frame(), time.data, coords, 
+           priors=NULL, initials=NULL, nItr, nBurn=0, report=1, 
+           tol.dist=2, distance.method="geodetic:km", cov.fnc="exponential",
+           scale.transform="NONE", spatial.decay,  truncation.para=NULL,
+		   X.out=TRUE, Y.out=TRUE)
 {
     start.time<-proc.time()[3]
+  #
   #
     if(nBurn >= nItr){
          stop(paste("\n Error: iterations < nBurn\n Here, nBurn = ",nBurn," and iterations = ",nItr,"."))
     }
+  #  
   #  
     if (missing(formula)) {
          stop("\n Error: formula must be specified \n")
@@ -1312,7 +1342,7 @@ spGP.MCMC.Pred<-function(formula, data=parent.frame(), time.data,
     if (class(formula) != "formula") {
          stop("\n Error: equation must be in formula-class \n ...")
     }
-  #
+   #
          XY <- Formula.matrix(formula, data)
          Y <- XY[[1]]
          X <- as.matrix(XY[[2]])
@@ -1321,52 +1351,42 @@ spGP.MCMC.Pred<-function(formula, data=parent.frame(), time.data,
          x.names.sp <- XY[[5]]
          Xtp <- XY[[6]]
          x.names.tp <- XY[[7]]
-  #
-   if(length(x.names.sp)>0){ stop("\n Error: currently not available for spatially varying coefficient process model. \n")} 
-   if(length(x.names.tp)>0){ stop("\n Error: currently not available for spatio-temporal DLM. \n")} 
-  #
-    if (missing(coords)) {
-         stop("Error: need to specify the coords")
-    }
-    if ( !is.matrix(coords) ) {
-         stop("\n Error: coords must be a (n x 2) matrix of xy-coordinate locations \n")
-    }
-    if ( (!is.numeric(coords[,1])) | (!is.numeric(coords[,2]))) {
-         stop("\n Error: coords columns should be numeric \n")
-    }
-    #
-      method <- distance.method
-      spT.check.sites.inside(coords, method, tol=tol.dist)
+
    #
-    if(method=="geodetic:km"){
-      coords.D <- as.matrix(spT.geodist(Lon=coords[,1],Lat=coords[,2], KM=TRUE))
+    if (missing(coords)) {
+         stop("\n Error: need to specify the coords \n")
     }
-    else if(method=="geodetic:mile"){
-      coords.D <- as.matrix(spT.geodist(Lon=coords[,1],Lat=coords[,2], KM=FALSE))
+    if ( !is.matrix(coords) & !is.data.frame(coords)) {
+         stop("\n Error: coords must be a (n x 2) matrix or data frame of xy-coordinate locations \n")
     }
-    else {
-       coords.D <- as.matrix(dist(coords, method, diag = T, upper = T))
-    }  
+    if ( dim(coords)[[2]] !=2) {
+         stop("\n Error: coords should have 2 columns \n")
+    }
+   #
+     method <- distance.method
+     spT.check.sites.inside(coords, method, tol=tol.dist)
+   #
+   if(method=="geodetic:km"){
+     coords.D <- as.matrix(spT.geodist(Lon=coords[,1],Lat=coords[,2], KM=TRUE))
+   }
+   else if(method=="geodetic:mile"){
+     coords.D <- as.matrix(spT.geodist(Lon=coords[,1],Lat=coords[,2], KM=FALSE))
+   }
+   else{
+     coords.D <- as.matrix(dist(coords, method, diag = TRUE, upper = TRUE))
+   }
    #
    # check time.data
    if(is.null(time.data)){
-     #time.data<-c(1,0,length(Y)/length(coords[,1]))
      time.data<-list(1,length(Y)/length(coords[,1]))
    }
    else{
      time.data<-time.data
    }
    #
-         n <- length(coords[,1])            # number of sites
+         n <- length(coords[,1])              # number of sites
          r <- time.data[[1]]                  # number of years
          T <- time.data[[2]]                  # number of days
-         # check for T
-         if(r > 1){ 
-            if(length(T) != r){         
-              T<-rep(T,r) 
-            }
-         }
-         #  
          # checking unequal T
          if(length(T) > 1){
            rT <- sum(T)
@@ -1375,32 +1395,33 @@ spGP.MCMC.Pred<-function(formula, data=parent.frame(), time.data,
            rT <- r*T
          }
          N <- n*rT
-   #
-    if (N != length(Y)) {
-         stop(" Error: Years, Months, and Days are misspecified,\n i.e., total number of observations in the data set should be equal to N\n  : N = n * r * T \n   where, N = total number of observations in the data,\n          n = total number of sites,\n          r = total number of years,\n          T = total number of days.\n# Check the function spT.time.\n#\n")
-    }
     #
-         p <- length(x.names)          # number of covariates
+    if (N != length(Y)) {
+         stop(" Error: Years, Months, and Days are misspecified,\n i.e., total number of observations in the data set should be equal to N\n  : N = n * r * T \n   where, N = total number of observations in the data,\n          n = total number of sites,\n          r = total number of years,\n          T = total number of days. \n## Check spT.time function.")
+    }
     #
          priors<-priors.checking.gp(priors)
     #
          shape_e<-N/2+priors$prior_a
          shape_eta<-N/2+priors$prior_a
     #
-         zm <- matrix(Y,rT,n)
-         zm <- apply(zm,1,median,na.rm=TRUE)
-         zm <- rep(zm,n)
-         zm <- cbind(Y,zm)
-         zm[is.na(zm[,1]),1]=zm[is.na(zm[,1]),2]
-         zm <- zm[,1]
-         names(zm)<-NULL
+	if(is.null(truncation.para)){
+	   stop(" Error: define truncation parameter lambda and truncation point \n")
+	}
+	     at <- truncation.para$at
+		 lambda <- truncation.para$lambda
+ 	     zm <- truncated.fnc(Y, at=at, lambda=lambda, both=FALSE)
+         zmm <- matrix(zm,rT,n)
+         zmm <- apply(zmm,1,median,na.rm=TRUE)
+         zmm <- rep(zmm,n)
+         zm <- cbind(zm,zmm)
+         zm[is.na(zm[,1]),1] <- zm[is.na(zm[,1]),2]
+         zm[is.na(zm[,1]),1] <- median(zm[,2],na.rm=TRUE)
     #
          flag <- matrix(NA,n*rT,2)
          flag[,1] <- c(Y)
          flag[!is.na(flag[,1]),2] <- 0
          flag[is.na(flag[,1]),2] <- 1
-         flag <- flag[,2]
-    #
     #
     if(cov.fnc=="exponential"){
          cov <- 1
@@ -1418,28 +1439,22 @@ spGP.MCMC.Pred<-function(formula, data=parent.frame(), time.data,
          stop("\n Error: cov.fnc is not correctly specified \n")
     }
     #
-    if (scale.transform == "NONE") {
-         zm <- zm
-         trans <- 0
-         scale.transform = "NONE"
+    if(scale.transform=="NONE"){
+         zm[,1] <- zm[,1]
     }
-    else if (scale.transform == "SQRT") {
-         zm <- sqrt(zm)
-         trans <- 1
-         scale.transform = "SQRT"
+    else if(scale.transform=="SQRT"){
+         zm[,1] <- sqrt(zm[,1])
     }
-    else if (scale.transform == "LOG") {
-         zm <- log(zm)
-         trans <- 2
-         scale.transform = "LOG"
+    else if(scale.transform=="LOG"){
+         zm[,1] <- log(zm[,1])
     }
     else{
          stop("\n Error: scale.transform is not correctly specified \n")
     }
     #
-      initials<-initials.checking.gp(initials,zm,X,Xsp,Xtp,n,r,T,coords.D)
+      initials<-initials.checking.gp(initials,zm[,1],X,Xsp,Xtp,n,r,T,coords.D)
     #
-         o <- zm
+         o <- zm[,1]
     #
     if(spatial.decay$type=="FIXED"){
          spdecay <- 1
@@ -1448,7 +1463,7 @@ spGP.MCMC.Pred<-function(formula, data=parent.frame(), time.data,
 		 }
          init.phi <- spatial.decay$value 
          tuning <- 0; phis<-0; phik<-0; 
-		 phi_a<-0; phi_b<-0
+		 phi_a <- 0; phi_b <- 0; 
     }
     else if(spatial.decay$type=="DISCRETE"){
          spdecay <- 2
@@ -1463,198 +1478,279 @@ spGP.MCMC.Pred<-function(formula, data=parent.frame(), time.data,
          init.phi <- initials$phi 
          tuning <- spatial.decay$tuning
          phis<-0; phik<-0; 
- 		 phi_a<-spatial.decay$val[1]
+		 phi_a<-spatial.decay$val[1]
 		 phi_b<-spatial.decay$val[2]
     }
     else{
          stop("\n Error: spatial.decay is not correctly specified \n")
     }
+    # 
+         p <- length(x.names)          # number of covariates
     #
-      if (length(initials$beta) != p){
+    if (length(initials$beta) != p){
          stop("Error: need to specify correct number of parameters for beta.")
-      }
+    }
     #
-    # prediction part
-    #
-        if (missing(pred.coords)) {
-           stop("Error: need to specify the prediction coords.")
-        }
-        if (!is.matrix(pred.coords)) {
-           stop("Error: prediction coords must be a (n x 2) matrix of xy-coordinate locations.")
-        }
-        if ( (!is.numeric(pred.coords[,1])) | (!is.numeric(pred.coords[,2]))) {
-           stop("\n Error: prediction coords columns should be numeric \n")
-        }
-      #
-      #
-           coords.all <- rbind(coords,pred.coords)
-           spT.check.locations(coords, pred.coords, method, tol=tol.dist)
-           tn.fitsites <- length(coords[, 1])
-           nfit.sites <- 1:tn.fitsites
-           tn.predsites <- length(coords.all[, 1]) - tn.fitsites
-           npred.sites <- (tn.fitsites + 1):(length(coords.all[, 1]))
-     #                                                                         
-      if(method=="geodetic:km"){
-           coords.D.all <- as.matrix(spT.geodist(Lon=coords.all[,1],Lat=coords.all[,2], KM=TRUE))
-      }
-      else if(method=="geodetic:mile"){
-           coords.D.all <- as.matrix(spT.geodist(Lon=coords.all[,1],Lat=coords.all[,2], KM=FALSE))
-      }
-      else {
-           coords.D.all<- as.matrix(dist(coords.all, method, diag = T, upper = T))
-      }
-      #
-           d12 <- coords.D.all[nfit.sites, npred.sites]
-           dns <- coords.D.all[(tn.fitsites+1):length(coords.all[,1]), (tn.fitsites+1):length(coords.all[,1])]
-      #
-           nsite <- tn.predsites
-           predN <- nsite*rT
-      #
-    #
-    # adding the formula in to the prediction dataset
-      if(!is.data.frame(pred.data) & !is.null(pred.data)){
-        stop("#\n# Error: pred.data should be in data format\n#")
-      }
-      call.f<-formula  
-      call.f<-as.formula(paste("tmp~",paste(call.f,sep="")[[3]]))
-      if(is.data.frame(pred.data)){
-      if((nsite*rT)!=dim(pred.data)[[1]]){
-        print("#\n # Check the pred.data \n#\n")
-      }
-      pred.data$tmp<-rep(1,nsite*rT)
-      }
-      if(is.null(pred.data)){
-        pred.data<-data.frame(tmp=rep(1,nsite*rT))
-      }
-      pred.x<-Formula.matrix(call.f,data=pred.data)[[2]]
-    #
-    #
-       if(annual.aggregation=="NONE"){
-         aggtype<-0
-       }
-       else if(annual.aggregation=="ave"){
-         aggtype<-1
-       }
-       else if(annual.aggregation=="an4th"){
-         aggtype<-2
-       }
-       else if(annual.aggregation=="w126"){
-         aggtype<-3
-         if(T != 214){
-          stop("\n# Error: day/time should have 214 values for annual.aggregation = w126.\n")        
+    if((length(x.names.sp) == 0) & (length(x.names.tp) == 0)){
+    # non-spatial and non-temporal beta
+    # check for T
+      if(r > 1){ 
+         if(length(T) != r){         
+           T<-rep(T,r) 
          }
+      }
+    #  
+    out<-.C('GIBBS_gp',as.double(flag[,2]),as.integer(nItr),
+         as.integer(nBurn),as.integer(n),as.integer(T),
+         as.integer(r),as.integer(rT),as.integer(p),
+         as.integer(N),as.integer(report),as.integer(cov),
+         as.integer(spdecay),as.double(shape_e),
+         as.double(shape_eta),
+		 as.double(phi_a), as.double(phi_b),
+		 as.double(priors$prior_a),
+         as.double(priors$prior_b),as.double(priors$prior_mubeta),    
+         as.double(priors$prior_sigbeta),as.double(priors$prior_omu), 
+         as.double(priors$prior_osig),as.double(init.phi),
+         as.double(tuning),as.double(phis),as.integer(phik),
+         as.double(coords.D),
+         as.double(initials$sig2eps), as.double(initials$sig2eta),
+         as.double(initials$beta),as.double(X),as.double(zm[,1]),  
+         as.double(o),as.integer(1),phip=double(nItr),
+         accept=double(1),nup=double(nItr),sig2ep=double(nItr),
+         sig2etap=double(nItr),betap=matrix(double(p*nItr),p,nItr), 
+         op=matrix(double(N*nItr),N,nItr),fit=matrix(double(N*2),N,2), 
+         gof=double(1),penalty=double(1))[35:44]
+    }
+    else if((length(x.names.sp) > 0) & (length(x.names.tp) == 0)){
+    # for spatial beta
+	if(length(T)> 1){ stop("\n ## Error: Unequal T_l is currently not possible for spatially varying coefficient process model. ##\n")}
+    q <- length(x.names.sp)          # number of spatial covariates
+    shape_beta<-(n*q)/2+priors$prior_a
+    if("(Intercept)" %in% x.names){
+       if(sum(c(X[,1])) == 0){
+       intercept <- 0
+       }     
+       else{ intercept <- 1 }
+    }
+    else{ 
+       intercept <- 1 
+    }
+    out<-.C("GIBBSsp_gp",as.integer(intercept),as.double(flag[,2]),as.integer(nItr),as.integer(nBurn),
+         as.integer(n),as.integer(T),as.integer(r),as.integer(rT),as.integer(p),
+         as.integer(q),as.integer(N),as.integer(report),as.integer(cov),
+         as.integer(spdecay),as.double(shape_e),as.double(shape_eta),as.double(shape_beta),   
+         as.double(priors$prior_a),as.double(priors$prior_b),as.double(priors$prior_mubeta), 
+         as.double(priors$prior_sigbeta),as.double(priors$prior_omu),as.double(priors$prior_osig),
+         as.double(init.phi),as.double(tuning),as.double(phis),as.integer(phik),
+         as.double(coords.D),as.double(initials$sig2eps),as.double(initials$sig2eta),as.double(initials$sig2beta), 
+         as.double(initials$beta),as.double(initials$betas),as.double(X),as.double(Xsp),
+         as.double(zm[,1]),as.double(o),as.integer(1), 
+         phip=double(nItr), accept=double(1),nup=double(nItr),sig2ep=double(nItr),sig2etap=double(nItr),  
+         sig2betasp=double(nItr),betap=matrix(double(nItr*p),p,nItr),betasp=matrix(double(nItr*n*q),n*q,nItr),
+         op=matrix(double(N*nItr),N,nItr),fit=matrix(double(N*2),N,2),gof=double(1),penalty=double(1))[39:50]
+    } 
+    else if((length(x.names.sp) == 0) & (length(x.names.tp) > 0)){
+    # for temporal beta
+    if(length(T)> 1){ stop("\n ## Error: Unequal T_l is currently not possible for spatio-temporal DLM. ##\n")}
+    u <- length(x.names.tp)          # number of temporal covariates
+    shape_del<-(u*T)/2+priors$prior_a
+    shape_0<-u/2+priors$prior_a
+    if("(Intercept)" %in% x.names){
+       if(sum(c(X[,1])) == 0){
+       intercept <- 0
        }
-       else{
-        stop("Error: correctly define annual.aggregation.")
-       }         
+       else{ intercept <- 1 }     
+    }
+    else{ 
+       intercept <- 1 
+    }
+    if(is.null(initials$rhotp)){
+      stop("Error: need to provide indication for rho sampling")
+    }
+    out<-.C("GIBBStp_gp",as.integer(intercept),as.double(flag[,2]),as.integer(nItr),as.integer(nBurn),
+         as.integer(n),as.integer(T),as.integer(r),as.integer(rT),as.integer(p),
+         as.integer(u),as.integer(N),as.integer(report),as.integer(cov),as.integer(spdecay),as.integer(initials$rhotp),
+         as.double(shape_e),as.double(shape_eta),as.double(shape_del),as.double(shape_0),
+         as.double(priors$prior_a),as.double(priors$prior_b), 
+         as.double(priors$prior_mubeta),as.double(priors$prior_sigbeta),as.double(priors$prior_omu),
+         as.double(priors$prior_osig),
+         as.double(init.phi),as.double(tuning),as.double(phis),as.integer(phik),
+         as.double(coords.D),as.double(initials$sig2eps),as.double(initials$sig2eta),as.double(initials$sig2delta),
+         as.double(0.01),as.double(initials$beta),as.double(initials$betat),as.double(rep(1,u)),
+         as.double(X),as.double(Xtp),as.double(zm[,1]),as.double(o),as.integer(1), 
+         phip=double(nItr),accept=double(1),nup=double(nItr),sig2ep=double(nItr),sig2etap=double(nItr),  
+         sig2deltap=double(nItr),sig2op=double(nItr),rhotp=matrix(double(u*nItr),u,nItr),
+         betap=matrix(double(nItr*p),p,nItr),betat0p=matrix(double(u*nItr),u,nItr),betatp=matrix(double(nItr*u*T),u*T,nItr), 
+         op=matrix(double(N*nItr),N,nItr),fit=matrix(double(N*2),N,2),gof=double(1),penalty=double(1))[43:57]
+    }
+    else if((length(x.names.sp) > 0) & (length(x.names.tp) > 0)){
+    # for both spatial and temporal beta
+    if(length(T)> 1){ stop("\n ## Error: Unequal T_l is currently not possible for spatio-dynamic model.## \n")}
+    q <- length(x.names.sp)          # number of spatial covariates
+    u <- length(x.names.tp)          # number of temporal covariates
+    shape_beta<-(n*q)/2+priors$prior_a
+    shape_del<-(u*T)/2+priors$prior_a
+    shape_0<-u/2+priors$prior_a
+    if(is.null(initials$rhotp)){
+      stop("Error: need to provide indication for rho sampling")
+    }
+    if("(Intercept)" %in% x.names){
+       if(sum(c(X[,1])) == 0){
+       intercept <- 0
+       }
+       else{ intercept <- 1 }     
+    }
+    else{ 
+       intercept <- 1 
+    }
+    out<-.C("GIBBSsptp_gp",as.integer(intercept), as.double(flag[,2]),as.integer(nItr),as.integer(nBurn),
+         as.integer(n),as.integer(T),as.integer(r),as.integer(rT),as.integer(p),
+         as.integer(q),as.integer(u),as.integer(N),as.integer(report),as.integer(cov),as.integer(spdecay),as.integer(initials$rhotp),
+         as.double(shape_e),as.double(shape_eta),as.double(shape_beta),as.double(shape_del),as.double(shape_0),
+         as.double(priors$prior_a),as.double(priors$prior_b),
+         as.double(priors$prior_mubeta),as.double(priors$prior_sigbeta),as.double(priors$prior_omu),as.double(priors$prior_osig), 
+         as.double(init.phi),as.double(tuning),as.double(phis),as.integer(phik),
+         as.double(coords.D),as.double(initials$sig2eps),as.double(initials$sig2eta),as.double(initials$sig2beta),
+         as.double(initials$sig2delta),as.double(0.1),
+         as.double(initials$beta),as.double(initials$betas),
+         as.double(initials$betat),as.double(rep(1,u)),as.double(X),as.double(Xsp),as.double(Xtp), 
+         as.double(zm[,1]),as.double(o),as.integer(1),
+         phip=double(nItr),accept=double(1),nup=double(nItr),sig2ep=double(nItr),sig2etap=double(nItr),  
+         sig2betasp=double(nItr),sig2deltap=double(nItr),sig2op=double(nItr),rhotp=matrix(double(u*nItr),u,nItr),
+         betap=matrix(double(nItr*p),p,nItr),betasp=matrix(double(nItr*n*q),n*q,nItr),
+         betat0p=matrix(double(u*nItr),u,nItr),betatp=matrix(double(nItr*u*T),u*T,nItr), 
+         op=matrix(double(N*nItr),N,nItr),fit=matrix(double(N*2),N,2),gof=double(1),penalty=double(1))[48:64]
+    }
+    else{
+         stop("\n#\n## Error: \n#")
+    }
     #
-      #
-      ########
-      #
-        out <- NULL
-        out <- .C("GIBBS_sumpred_txt_gp", as.integer(aggtype),as.double(flag), as.integer(nItr),
-            as.integer(nBurn), as.integer(n), as.integer(T), as.integer(r), 
-            as.integer(rT), as.integer(p), as.integer(N), as.integer(report),
-            as.integer(cov), as.integer(spdecay), as.double(shape_e), as.double(shape_eta),
-            as.double(phi_a), as.double(phi_b),			
-            as.double(priors$prior_a), as.double(priors$prior_b), 
-            as.double(priors$prior_mubeta), as.double(priors$prior_sigbeta),
-            as.double(priors$prior_omu), as.double(priors$prior_osig),
-            as.double(init.phi), as.double(tuning),
-            as.double(phis), as.integer(phik), as.double(coords.D), 
-            as.double(initials$sig2eps), as.double(initials$sig2eta),
-            as.double(initials$beta), as.double(X), as.double(zm), as.double(o),
-            as.integer(1), 
-            as.integer(nsite), as.integer(predN), as.double(d12), as.double(pred.x), 
-            as.integer(trans), accept=double(1), gof=double(1),penalty=double(1))[41:43] 
-     #
-      out$accept <- round(out$accept/nItr*100,2)
-      out$call<-formula
-     #
-      cat("##","\n")
-      cat("# MCMC output and predictions are given in text format ... \n")
-      cat("# nBurn = ",nBurn,". Iterations = ",nItr,".", "\n")
-      cat("# Acceptance rate: (phi) = ",out$accept,"%", "\n")
-      cat("##","\n")
-     #
-     #
-           tmp<-read.table('OutGP_Values_Parameter.txt',sep='',header=FALSE)
-           tmp<-tmp[(nBurn+1):nItr,]
-           tmp<-spT.Summary.Stat(t(tmp))
+      accept <- round(out$accept/nItr*100,2)
+    #
+      output <- NULL
+    #
+     if(X.out==TRUE){
+        if((length(x.names.sp) == 0) & (length(x.names.tp) == 0)){
+        # non-spatial and temporal beta
+          output$X <- X
+          #dimnames(output$X)[[2]] <- x.names
+        }
+        else if((length(x.names.sp) > 0) & (length(x.names.tp) == 0)){
+        # for spatial beta
+          output$X <- X
+          output$Xsp <- Xsp
+          dimnames(output$X)[[2]] <- as.list(x.names)
+          #dimnames(output$Xsp)[[2]] <- x.names.sp
+        }  
+        else if((length(x.names.sp) == 0) & (length(x.names.tp) > 0)){
+        # for temporal beta
+          output$X <- X
+          output$Xtp <- Xtp
+          dimnames(output$X)[[2]] <- as.list(x.names)
+          #dimnames(output$Xtp)[[2]] <- x.names.tp
+        }
+        else if((length(x.names.sp) > 0) & (length(x.names.tp) > 0)){
+        # for both spatial and temporal beta
+          output$X <- X
+          output$Xsp <- Xsp
+          output$Xtp <- Xtp
+          dimnames(output$X)[[2]] <- as.list(x.names)
+          #dimnames(output$Xsp)[[2]] <- x.names.sp
+          #dimnames(output$Xtp)[[2]] <- x.names.tp
+        }
+        else{
+          stop("\n#\n## Error: \n#")
+        }
+     }
+     if(Y.out==TRUE){
+        output$Y <- Y
+     }
+    #
+      output$accept <- accept
+      output$call <- formula
+    #
+           output$phip <- as.matrix(out$phip[(nBurn+1):nItr])
            if(cov==4){
-           row.names(tmp)<- c(x.names,"sig2eps","sig2eta","phi","nu")
+           output$nup <- as.matrix(out$nup[(nBurn+1):nItr])
            }
-           else{
-           row.names(tmp)<- c(x.names,"sig2eps","sig2eta","phi")
-           } 
-           out$parameters<-round(tmp, 4)
-           tmp<-NULL
-     #
-           tmp<-read.table('OutGP_Stats_FittedValue.txt', sep=',',header=FALSE)
-           names(tmp)<- c("Mean","SD")
-           out$fitted<-round(tmp, 4)
-           tmp<-NULL
-     #
-           tmp<-read.table('OutGP_Stats_PredValue.txt', sep=',',header=FALSE)
-           names(tmp)<- c("Mean","SD")
-           out$prediction<-round(tmp, 4)
-           tmp<-NULL
-     #
-       if(annual.aggregation=="ave"){
-           tmp<-read.table('OutGP_Annual_Average_Prediction.txt', sep='',header=FALSE)
-           tmp<-spT.Summary.Stat(t(tmp))
-           out$an.agr.pred.average<-round(tmp, 4)
-           tmp<-NULL
-       }   
-       if(annual.aggregation=="an4th"){
-           tmp<-read.table('OutGP_Annual_4th_Highest_Prediction.txt', sep='',header=FALSE)
-           tmp<-spT.Summary.Stat(t(tmp))
-           out$an.agr.pred.an4th<-round(tmp, 4)
-           tmp<-NULL
-       }   
-       if(annual.aggregation=="w126"){
-           tmp<-read.table('OutGP_Annual_w126_Prediction.txt', sep='',header=FALSE)
-           tmp<-spT.Summary.Stat(t(tmp))
-           out$an.agr.pred.w126<-round(tmp, 4)
-           tmp<-NULL
-       }   
-     #
-           out$tol.dist <- tol.dist	
-           out$distance.method <- distance.method
-           out$cov.fnc <- cov.fnc
-           out$scale.transform <- scale.transform
-           out$sampling.sp.decay<-spatial.decay
-           out$covariate.names<-x.names
-           out$gof<-round(out$gof,2)
-           out$penalty<-round(out$penalty,2) 
-           tmp <- matrix(c(out$gof,out$penalty,out$gof+out$penalty),1,3)
-           dimnames(tmp)[[2]] <- c("Goodness.of.fit","Penalty","PMCC")
-           dimnames(tmp)[[1]] <- c("values:")
-           out$PMCC <- tmp
+           output$sig2ep <- as.matrix(out$sig2ep[(nBurn+1):nItr])
+           output$sig2etap <- as.matrix(out$sig2etap[(nBurn+1):nItr])
+           if(length(x.names.sp) != 0){          
+           output$sig2betap <- as.matrix(out$sig2betasp[(nBurn+1):nItr])
+           }
+           if(length(x.names.tp) != 0){          
+           output$sig2deltap <- as.matrix(out$sig2deltap[(nBurn+1):nItr])
+           output$sig2op <- as.matrix(out$sig2op[(nBurn+1):nItr])
+           }
+           output$betap <- matrix(out$betap[1:p,(nBurn+1):nItr],p,length((nBurn+1):nItr))
+           if(length(x.names.sp) != 0){          
+           output$betasp <- matrix(out$betasp[1:(n*q),(nBurn+1):nItr],n*q,length((nBurn+1):nItr))
+           }
+           if(length(x.names.tp) != 0){    
+           output$betat0p <- matrix(out$betat0p[1:u,(nBurn+1):nItr],u,length((nBurn+1):nItr))
+           output$betatp <- matrix(out$betatp[1:(u*T),(nBurn+1):nItr],u*T,length((nBurn+1):nItr))
+           output$rhotp <- matrix(out$rhotp[1:u,(nBurn+1):nItr],u,length((nBurn+1):nItr))
+           }
+           output$op <- out$op[1:N,(nBurn+1):nItr]
+           output$wp <- output$op-output$X%*%output$betap
+		   fit.val <- reverse.truncated.fnc(output$op,at=at,lambda=lambda)
+		   output$fitted <- cbind(apply(fit.val,1,median),prob.below.threshold(fit.val,at=at))
+           dimnames(output$fitted)[[2]] <- c("Median","Prob.below.threshold")
+           output$tol.dist<-tol.dist
+           output$distance.method<-method
+           output$cov.fnc<-cov.fnc
+           output$scale.transform<-scale.transform
+           output$sampling.sp.decay<-spatial.decay
+           output$covariate.names<-x.names
+           if(length(x.names.sp) != 0){          
+           output$sp.covariate.names<-c(x.names.sp)
+           }
+           if(length(x.names.tp) != 0){          
+           output$tp.covariate.names<-c(x.names.tp)
+           }
+           output$Distance.matrix <- coords.D
+           output$coords <- coords
+           output$n <- n
+           output$r <- r
+           output$T <- T
+           output$p <- p
+           if(length(x.names.sp) != 0){          
+           output$q <- q
+           }
+           if(length(x.names.tp) != 0){          
+           output$u <- u
+           }
+           output$initials <- initials	
+           output$priors <- priors	
+           output$gof <- round(out$gof,2)
+           output$penalty <- round(out$penalty,2)
+           tmp <- matrix(c(output$gof,output$penalty,output$gof+output$penalty),1,3)
+           dimnames(tmp)[[2]]<-c("Goodness.of.fit","Penalty","PMCC")
+           dimnames(tmp)[[1]]<-c("values:")
+           output$PMCC <- tmp
            tmp <- NULL
-           out$gof<-NULL
-           out$penalty<-NULL 
-           out$n <- n
-           out$pred.n<-nsite
-           out$r <- r
-           out$T <- T
-           out$Y <- Y
-           out$initials <- initials	
-           out$priors <- priors 
-           out$iterations <- nItr 
-           out$nBurn <- nBurn 
-      #
+           output$gof <- NULL
+           output$penalty <- NULL
+    #
+           output$iterations <- nItr	
+           output$nBurn <- nBurn	
+    #
+     rm(out)
+    #
+      cat("##","\n")
+      cat("# nBurn = ",nBurn,", Iterations = ",nItr,".", "\n")
+      cat("# Overall Acceptance Rate (phi) = ",output$accept,"%", "\n")
+      cat("##","\n")
     #
    end.time <- proc.time()[3]
    comp.time<-end.time-start.time
    comp.time<-fnc.time(comp.time)
-   out$computation.time<-comp.time
+   output$computation.time<-comp.time
+    #
+     #class(output) <- "spGP"
+     output
+    #
     #
-       #class(out) <- "spGP"
-      #
-       out
-      #
-      #
 }
 ##
 ##
